#
# The build script for ix86 PCs install disks
# (over-worked for dRock by Ren Rebe)
#

#global funtions
# create_fs_image src_dir block_count dest_file
create_fs_image () {
	echo "Creating (dumping) filesystem image ..."
	echo "  create_fs_image: $1 $2 $3 ..."
	
	if [ ".$1" == "." ] ; then
		echo "no source?"
		exit -1
	fi
	if [ ".$2" == "." ] ; then
		echo "no block count??"
		exit -1
	fi
	if [ ".$3" == "." ] ; then
		echo "no destination???"
		exit -1
	fi
	
	mkdir -p disk.mnt
	rm -rf disk.mnt/*
	
	dd if=/dev/zero of=$tmpdev bs=1k count=$2
	mke2fs -m0 $tmpdev $2 ; mount $tmpdev disk.mnt
	cp -a $1/* disk.mnt
	umount disk.mnt ; dd if=$tmpdev bs=1k count=$2 | gzip -9 > $3
	du -sh $3
	
	echo "  cleanup ..."
	rm -rf disk.mnt/*
	rm -rf $1/*
	echo "... finished"
}

create_wrap_link () {
	if [ -f $x ] ; then
                mv -fv $x $BIN_DIR;
                ln -s ../sbin/src-wrap $x
        else
                echo "WARNING: $x does not exist - (Maintainer should clean up!)"
        fi
}

error_clean_up () {
	echo "error_clean_up called"
	umount $tmpdev
	losetup -d $tmpdev
}

# free the loop device on error
trap error_clean_up EXIT QUIT TERM KILL

echo -e "\n **** Prepare creation of install floppies ****\n"

mkdir -p src/inst-disk
cd src/inst-disk
cp $sd_srcdir/intel/* .

echo "Create bin-dir in rock-trg-dir (for additional binaries) ..."
BIN_DIR=$base/rock-$arch-$rockver/bin
mkdir -p $BIN_DIR

dd if=/dev/zero of=buffer bs=1k count=4096 ; tmpdev=x
for x in /dev/loop/* ; do
	if [ $tmpdev = x ] ; then
		echo "Trying to use loopback dev $x ..."
		if losetup $x buffer ; then tmpdev=$x ; fi
	fi
done
if [ $tmpdev = x ] ; then
	echo "No free loopback device found. Abort." ; exit 1
fi
echo "Using loop device $tmpdev."
trap "losetup -d $tmpdev" 3 6 9 11 15

echo 'Extracting kernel and install-disk fs.'
tar xIf $sd_instdir/install-disks.tar.bz2
mv boot-disk/System.map .

echo 'Building custom binaries ...'

echo "  wget ..."
tar xfI $base/pkg-archive/wget-1.8.1.tar.bz2
cd wget-1.8.1
./configure  --without-ssl --disable-opie --disable-digest --disable-debug --disable-nls
make > /dev/null
strip src/wget ; cp -vf src/wget ../install-disk/bin/wget
rm -rf wget-1.8.1
cd ..

echo "  bash ..."
tar xfI $base/pkg-archive/bash-2.05a.tar.bz2
cd bash-2.05a
./configure --disable-help-builtin --disable-progcomp
make > /dev/null
strip bash ; cp -vf bash ../install-disk/bin/sh
cd ..

echo 'Creating temporary scratch point.'
mkdir disk.scratch ; mnt="`pwd`/disk.scratch"

echo 'Creating install-disk/sbin/modules'

cat > install-disk/sbin/modules << 'EOT'
#!/bin/sh

if [ ".$1" != ".pcmcia" -a ".$1" != ".net" -a ".$1" != ".scsi" ] ; then
	echo "Usage: $0 < pcmcia | net | scsi >"
	exit 1
fi

echo -n "Insert modules disk into /dev/floppy/0 and press [ENTER] .."
read x

if [ ! -d /lib/modules ] ; then
	mkdir -p /lib/modules
	mke2fs -vm0 /dev/rd/5 4096
	mount /dev/rd/5 /lib/modules
fi

mkdir -p /tmp/$$.mnt
mount -v -t ext2 -o ro /dev/floppy/0 /tmp/$$.mnt
cd /
tar xvfI /tmp/$$.mnt/mods-$1.tar.bz2
umount -v /dev/floppy/0
rmdir /tmp/$$.mnt
EOT
chmod +x install-disk/sbin/modules

echo -e "\n **** Create 'root-disk.img' ****\n"

cp -a install-disk/* $mnt/

echo "deleting some libraries due to missing space ..."
for x in $mnt/lib/libssl* $mnt/lib/libcrypto* ; do
	echo "del: $x"
	rm -vf $x
done

echo "moving/creating some the src-wrap space-saving ..."
for x in $mnt/bin/e2fsck $mnt/bin/tune2fs $mnt/bin/badblocks \
         $mnt/bin/sfdisk $mnt/bin/cfdisk \
         $mnt/bin/debugreiserfs $mnt/bin/reiserfsck ; do
        create_wrap_link $x
done
create_fs_image $mnt 4096 root-disk.img

echo -e "\n **** Create 'boot-disk.img' ****\n"
#
dd if=/dev/zero of=$tmpdev bs=1k count=1440
mke2fs -m0 $tmpdev 1440 ; mount $tmpdev $mnt
cp -a boot-disk/* lilo-help.txt $mnt/
ln -sf boot-text.b $mnt/boot.b
tr '\n=\t-' '=\n\-\t' < lilo-conf-1.txt |
sed 's,\\=-*,,' | tr '\n=\t-' '=\n\-\t' > $mnt/lilo.conf
sed -e "s,/mnt/,$mnt/,; s,/dev/floppy/0,$tmpdev,;" \
	< $mnt/lilo.conf > lilo.conf.tmp
boot-disk/lilo -C lilo.conf.tmp ; rm lilo.conf.tmp
umount $mnt ; dd if=$tmpdev of=boot-disk.img bs=1k count=1440

echo -e "\n **** Create 'modules-disk.img' ****\n"
#
if [ -f $sd_pkgdir/modules.tar.bz2 ]
then
	mkdir modules ; cd modules
	tar xIf $sd_pkgdir/modules.tar.bz2 lib/modules
	find lib/ -name "modules.*" | xargs rm -f
	find lib/ -type f | xargs strip --strip-debug --strip-unneeded
	
	echo "  creating PCMCIA package ..."
	tar cIf ../mods-pcmcia.tar.bz2  lib/modules/*/{,kernel/drivers/}pcmcia
	echo "  creating NET package ..."
	tar cIf ../mods-net.tar.bz2     lib/modules/*/kernel/drivers/net
	echo "  creating SCSI package ..."
	tar cIf ../mods-scsi.tar.bz2    lib/modules/*/kernel/drivers/scsi
	
	echo "  cleaup for min-SCSI package ..."
	# some are not in wide use, for the same device or for ISA cards
	rm -vrf lib/modules/*/kernel/drivers/scsi/{sg,st,sr_mod,sd_mod}.o
	rm -vrf lib/modules/*/kernel/drivers/scsi/{scsi_debug,osst}.o
	rm -vrf lib/modules/*/kernel/drivers/scsi/sym53c8xx_2
	rm -vrf lib/modules/*/kernel/drivers/scsi/{53c7\,8xx,sym53c416}.o
	rm -vrf lib/modules/*/kernel/drivers/scsi/{aic7xxx_old,NCR53c406a}.o
	rm -vrf lib/modules/*/kernel/drivers/scsi/{BusLogic,qlogicfc}.o
	rm -vrf lib/modules/*/kernel/drivers/scsi/{qlogicfas,qlogicisp}.o
	rm -vrf lib/modules/*/kernel/drivers/scsi/{ide-scsi,aha*}.o
	rm -vrf lib/modules/*/kernel/drivers/scsi/{AM53C974,eata*}.o
	rm -vrf lib/modules/*/kernel/drivers/scsi/{Aqla1280,g_NCR5380}.o
	echo "  creating min-SCSI package ..."
	tar cvfI ../min-mods-scsi.tar.bz2  lib/modules/*/kernel/drivers/scsi
	
	cd .. ; rm -rf modules
	
	echo "  creating modules-disk.img ..."
	cp mods-pcmcia.tar.bz2 mods-net.tar.bz2 min-mods-scsi.tar.bz2 $mnt/
        create_fs_image $mnt 1440 modules-disk.img
fi

echo -e "\n **** Create 'root-kombo.img' ****\n"

cp -a install-disk/* $mnt/

echo "deleting some libraries due to missing space ..."
for x in $mnt/lib/libssl* $mnt/lib/libcrypto* ; do
        rm -vf $x
done

echo "moving / creating some the src-wrap space-saving ..."
for x in $mnt/bin/e2fsck $mnt/bin/tune2fs $mnt/bin/badblocks \
         $mnt/bin/route \
         $mnt/bin/sfdisk $mnt/bin/cfdisk \
         $mnt/bin/debugreiserfs $mnt/bin/reiserfsck ; do
	create_wrap_link $x
done
rm -rf $mnt/lib/libnss_files[.-]* $mnt/etc/hosts $mnt/usr/share/terminfo/[!l]

if [ -f min-mods-scsi.tar.bz2 ] ; then
	cp min-mods-scsi.tar.bz2 $mnt/min-mods-scsi.tar.bz2
	cat > $mnt/etc/setup-mods-scsi.sh <<- EOT
		echo 'Extracting SCSI modules ...'
		mkdir -p /lib/modules ; mke2fs -q /dev/rd/5
		mount -v /dev/rd/5 -t ext2 /lib/modules
		tar xIf /min-mods-scsi.tar.bz2 -C /
		rm /min-mods-scsi.tar.bz2
		du -s /lib/modules
	EOT
fi

create_fs_image $mnt 4096 root-kombo.img

echo -e "\n **** Create 'kombo-disk.img' ****\n"

dd if=/dev/zero of=$tmpdev bs=1k count=2880
mke2fs -m0 $tmpdev 2880 ; mount $tmpdev $mnt
cp -v root-kombo.img boot-disk/vmlinuz lilo-help.txt $mnt/
cp -v boot-disk/memtest86 $mnt/
cp -v boot-disk/boot-text.b $mnt/boot.b
tr '\n=\t-' '=\n\-\t' < lilo-conf-2.txt | \
sed 's,\\=-*,,' | tr '\n=\t-' '=\n\-\t' | grep -v '#' > $mnt/lilo.conf
sed -e "s,/mnt/,$mnt/,; s,/dev/floppy/0,$tmpdev,;" \
	< $mnt/lilo.conf > lilo.conf.tmp
boot-disk/lilo -C lilo.conf.tmp ; rm lilo.conf.tmp
umount $mnt ; dd if=$tmpdev of=kombo-disk.img bs=1k count=2880

echo -e "\n **** Finish and Cleanup ****\n"

mkdir demo-setup ; mmv setup-\* demo-setup/\#1
tar cf demo-setup.tar demo-setup

mkdir -p $base/rock-$arch-$rockver/floppies/
mv System.map boot-disk/vmlinuz *.img boot-disk/memtest* demo-setup.tar \
	$base/rock-$arch-$rockver/floppies/
cp $base/arch-conf/$arch/kernel.conf $sd_srcdir/intel/README \
	mk-[br]ootdisk.sh $base/rock-$arch-$rockver/floppies/
chmod +x $base/rock-$arch-$rockver/floppies/mk-[br]ootdisk.sh
cd $base ; losetup -d $tmpdev ; rm -rf src/inst-disk
